#!/usr/bin/python
# -*- coding: UTF-8 -*-

# 28.08.2013

# Kompass mit einstellbarer Kompassrose.
# Nach dem Initialisieren ist die Kompassrose unsichtbar.
# Erst nach dem ersten rotate() wird sie dargestellt.

from tkinter import *
from string import *
from math import sin, cos, pi
import time
import _thread

back_grd       = "#F0F0F0"
compass_bg     = "#808080"        # Hintergrund hinter dem Kreis
compass_rose   = "#C0C0C0"        #

class Compass(object):
    """ Kompass mit 15° Teilung und Kompassnadel """

    def __init__(self, master,
                 width       = "600",           # Ausdehnung in X/Y-Richtung in Pixeln
                 titel       = "Kompass",         # Beschriftung des Frames
                 row         = 0,               # Plazierung im übergeordneten Frame
                 column      = 0,
                 **kw):

        self.__frm1 = LabelFrame(master, text = titel) #, width = width, height = width)
        self.__frm1.grid(column = column, row = row, padx = 10, pady = 10)
        #self.__frm1.grid_propagate(0)

        self.__cv1  = Canvas(self.__frm1, width = width, height = width)
        self.__cv1.grid(column = 0, row = 0, padx = 10, pady = 10)

        self.__oval = self.__cv1.create_oval("3","3", width, width,
                                          fill = compass_rose, outline = compass_rose)
        self.mid = 3 + int(width) / 2

        # Die Skalenteilung
        self.set_scale()

        # Beschriftung der Himmelsrichtungen
        self.__cv1.create_text(str(self.mid), "50", text = "N", font = ('Helvetica', '30', 'bold'), anchor = "center")
        self.__cv1.create_text(str(self.mid), "550", text = "S", font = ('Helvetica', '30', 'bold'), anchor = "center")
        self.__cv1.create_text("60", str(self.mid), text = "W", font = ('Helvetica', '30', 'bold'), anchor = "center")
        self.__cv1.create_text("550", str(self.mid), text = "E", font = ('Helvetica', '30', 'bold'), anchor = "center")

        # Polygone zuerst nur als Punkte zeichnen
        # das erste Rotate() bringt sie zum Vorschein
        self.rose1 = self.__cv1.create_polygon(str(self.mid),str(self.mid), str(self.mid), str(self.mid), str(self.mid), str(self.mid), width = 3, outline = "#202020", fill = "#202020")
        self.rose2 = self.__cv1.create_polygon(str(self.mid),str(self.mid), str(self.mid), str(self.mid), str(self.mid), str(self.mid), width = 3, outline = "#202020", fill = '')

        # Die Abdeckung in der Mitte
        self.__cv1.create_oval(str(self.mid - 26), str(self.mid - 26), str(self.mid + 27), str(self.mid + 27), width = 2,
                        fill = "#000000", outline = "#FFFFFF")

    def set_scale(self):
        """ erzeugt die Skaleneinteilung in 15° Schritten """
        for i in range(0, 361, 15):
            a = self.calc_angle(i)
            x1 = str(self.mid + a[0] * 275)
            y1 = str(self.mid + a[1] * 275)
            x2 = str(self.mid + a[0] * 295)
            y2 = str(self.mid + a[1] * 295)

            if i % 45 is 0:
                self.__cv1.create_line(x1, y1, x2, y2, width = 7, capstyle = ROUND)
            else:
                self.__cv1.create_line(x1, y1, x2, y2, width = 3, capstyle = ROUND)

    def rotate(self, w):
        """ Dreht die Kompassrose auf den gewünschten Winkel """
        w = 180 - w
        a = self.calc_angle(w)
        x3 = str(self.mid + a[0] * 295)
        y3 = str(self.mid + a[1] * 295)

        a = self.calc_angle(w - 90)
        x1 = str(self.mid + a[0] * 25)
        y1 = str(self.mid + a[1] * 25)

        a = self.calc_angle(w + 90)
        x2 = str(self.mid + a[0] * 25)
        y2 = str(self.mid + a[1] * 25)

        a = self.calc_angle(w + 180)
        x4 = str(self.mid + a[0] * 295)
        y4 = str(self.mid + a[1] * 295)

        self.__cv1.coords(self.rose1, x1, y1, x2, y2, x3, y3)
        self.__cv1.coords(self.rose2, x1, y1, x2, y2, x4, y4)

    def calc_angle(self, w):
        x =  sin( pi * (w / 180))
        y =  cos( pi * (w / 180) )
        return (x, y)


if __name__ == '__main__':

    def animate():
        try:
            for i in range(0, 361, 1):
                cp.rotate(i)
                time.sleep(.2)
        except:
            exit()

    top = Tk()
    top.title("Kompass")
    top.resizable(width = False, height = False) # Größe nicht veränderbar

    cp = Compass(top)

    _thread.start_new_thread(animate,())

    top.mainloop()
